@@ -34,7 +34,7 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.15'
+__version__ = '0.16'
 
 
 import ldap
@@ -98,7 +98,7 @@ class AuthenticationMethod(object):
     try:
       ldap_server.simple_bind_s(binddn, password)
       authenticated = True
-    except self.ldap_module.LDAPError, e:
+    except self.ldap_module.LDAPError:
       authenticated = False
     finally:
       ldap_server.unbind_s()
