@@ -35,13 +35,14 @@ Allows client to connect and run arbitrary functions in core.py.
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.15'
+__version__ = '0.16'
 
 
 import datetime
 import os
 import inspect
 import SocketServer
+import StringIO
 import time
 import traceback
 import uuid
@@ -54,6 +55,9 @@ from ssl_xml_rpc_lib import SecureXMLRPCServer
 from ssl_xml_rpc_lib import SecureXMLRpcRequestHandler
 
 
+roster_core.core.CheckCoreVersionMatches(__version__)
+
+
 class ArgumentError(roster_core.CoreError):
   pass
 
@@ -63,7 +67,7 @@ class FunctionError(roster_core.CoreError):
 class ServerError(roster_core.CoreError):
   pass
 
-class ThreadedXMLRPCServer(SocketServer.ThreadingMixIn,SecureXMLRPCServer):
+class ThreadedXMLRPCServer(SocketServer.ThreadingMixIn, SecureXMLRPCServer):
   pass
 
 class Server(object):
@@ -120,6 +124,7 @@ class Server(object):
     self.core_store = [] # {'user': user, 'last_used': last_used, 'instance': }
     self.get_credentials_wait = {} # {'user1': 3, 'user2': 4}
     self.last_cleaned = datetime.datetime.now()
+    self.LogMessage('Roster server started on port %s' % self.port, 'rosterd')
 
   def LogException(self, function, args, kwargs, user_name):
     """Save functions traceback to logfile
@@ -134,27 +139,56 @@ class Server(object):
       str: uuid string from logfile
     """
     uuid_string = str(uuid.uuid4())
-    if( not os.path.exists ):
-      try:
-        open(self.log_file, 'w').close()
-      except:
-        raise ServerError('Could not write to logfile "%s"' % self.log_file)
-    log_file_handle = open(self.log_file, 'a')
-    log_file_handle.writelines('\n\n---------------------\n')
-    log_file_handle.writelines(uuid_string)
-    log_file_handle.writelines('\n')
-    log_file_handle.writelines('FUNCTION: %s\n' % function)
-    log_file_handle.writelines('ARGS: %s\n' % args)
-    log_file_handle.writelines('KWARGS: %s\n' % kwargs)
-    log_file_handle.writelines('USER: %s\n' % user_name)
-    log_file_handle.writelines('TIMESTAMP: %s\n\n' % (
+    log_file_contents = []
+    exception_string = StringIO.StringIO()
+    traceback.print_exc(None, exception_string)
+    log_file_contents.append('\n\n---------------------\n')
+    log_file_contents.append(uuid_string)
+    log_file_contents.append('\n')
+    log_file_contents.append('FUNCTION: %s\n' % function)
+    log_file_contents.append('ARGS: %s\n' % args)
+    log_file_contents.append('KWARGS: %s\n' % kwargs)
+    log_file_contents.append('USER: %s\n' % user_name)
+    log_file_contents.append('TIMESTAMP: %s\n\n' % (
         datetime.datetime.now().isoformat()))
-    traceback.print_exc(None, log_file_handle)
-    log_file_handle.writelines('\n---------------------\n')
-    log_file_handle.close()
+    log_file_contents.append(exception_string.getvalue())
+    log_file_contents.append('\n---------------------\n')
+    try:
+      log_file_handle = open(self.log_file, 'a')
+      log_file_handle.writelines(''.join(log_file_contents))
+    finally:
+      log_file_handle.close()
 
     return uuid_string
 
+  def LogMessage(self, log_message, user_name):
+    """Save a message to the logfile
+
+    Inputs:
+      log_message: string of the log message
+      user_name: username string
+
+    Outputs:
+      str: uuid string from logfile
+    """
+    uuid_string = str(uuid.uuid4())
+    log_file_contents = []
+    log_file_contents.append('\n\n---------------------\n')
+    log_file_contents.append(uuid_string)
+    log_file_contents.append('\n')
+    log_file_contents.append('MESSAGE: %s\n' % log_message)
+    log_file_contents.append('USER: %s\n' % user_name)
+    log_file_contents.append('TIMESTAMP: %s' % (
+        datetime.datetime.now().isoformat()))
+    log_file_contents.append('\n---------------------\n')
+    try:
+      log_file_handle = open(self.log_file, 'a')
+      log_file_handle.writelines(''.join(log_file_contents))
+    finally:
+      log_file_handle.close()
+
+    return uuid_string
+  
   def CleanupCoreStore(self):
     """Cleans up expired instances in core_store"""
     delete_list = []
@@ -203,6 +237,7 @@ class Server(object):
     Outputs:
       instance: instance of dnsmgmtcore
     """
+    core_instance_dict = {}
     for core_instance_dict in self.core_store:
       if( core_instance_dict['user_name'] == user_name ):
         break
@@ -217,7 +252,7 @@ class Server(object):
 
     return core_instance_dict['core_instance']
 
-  def CoreRun(self, function, user_name, credfile, args=[], kwargs={}):
+  def CoreRun(self, function, user_name, credfile, args = None, kwargs = None):
     """Runs a function in core_instance with arbitrary parameters
 
     Inputs:
@@ -232,6 +267,10 @@ class Server(object):
                              'new_credential': u'
                                  be4d4ecf-d670-44a0-b957-770e118e2755'}
     """
+    if args is None:
+      args = []
+    if kwargs is None:
+      kwargs = {}
     uuid_string = None
     credfile = unicode(credfile)
     user_name = unicode(user_name)
@@ -261,6 +300,7 @@ class Server(object):
         raise FunctionError('Function does not exist.')
 
       types = inspect.getargspec(run_func)
+      error_class = None
       try:
         ## Figure out what each function is expecting
         if( types[3] is None and len(types[0]) == 1 ):
@@ -277,10 +317,17 @@ class Server(object):
           core_return = run_func(*args, **kwargs)
         else:
           raise ArgumentError('Arguments do not match.')
-      except Exception, e:
+      except roster_core.errors.InternalError, error:
         uuid_string = self.LogException(function, args, kwargs, user_name)
-        return {'log_uuid_string': uuid_string, 'error': str(e),
-                'core_return': None, 'new_credential': None}
+        error_class = 'InternalError'
+      except roster_core.errors.UserError, error:
+        error_class = 'UserError'
+      except Exception, error:
+        uuid_string = self.LogException(function, args, kwargs, user_name)
+        error_class = error.__class__.__name__
+      if( error_class is not None ):
+        return {'log_uuid_string': uuid_string, 'error': str(error),
+                'error_class': error_class, 'core_return': None, 'new_credential': None}
       core_return = {'core_return': core_return, 'new_credential': cred_status,
                      'log_uuid_string': uuid_string, 'error': None}
     else:
@@ -298,6 +345,7 @@ class Server(object):
       string: string of credential
               example: u'be4d4ecf-d670-44a0-b957-770e118e2755'
     """
+    error_class = None
     try:
       user_name = unicode(user_name)
       core_instance = self.GetCoreInstance(user_name)
@@ -311,10 +359,14 @@ class Server(object):
             user_name] + self.get_credentials_wait_increment
       elif( self.get_credentials_wait.has_key(user_name) ):
         self.get_credentials_wait.pop(user_name)
-    except Exception, e:
+    except Exception, error:
       uuid_string = self.LogException(
           'GetCredentials', [user_name, '<password>'], {}, user_name)
-      return {'log_uuid_string': uuid_string, 'error': str(e),
+      error_class = error.__class__.__name__
+      if( error_class is not None ):
+        return {'log_uuid_string': uuid_string, 'error': str(error),
+                'error_class': error_class, 'core_return': None, 'new_credential': None}
+      return {'log_uuid_string': uuid_string, 'error': str(error),
               'core_return': None, 'new_credential': None}
 
     return cred_string
@@ -328,21 +380,29 @@ class Server(object):
     Outputs:
       bool: bool of valid string
     """
+    error_class = None
     try:
       user_name = unicode(user_name)
       credstring = unicode(credstring)
       core_instance = self.GetCoreInstance(user_name)
       valid = self.cred_cache_instance.CheckCredential(
           credstring, user_name, core_instance)
-    except Exception, e:
+    except Exception, error:
       uuid_string = self.LogException(
           'IsAuthenticated', [user_name, '<credstring>'], {}, user_name)
-      return {'log_uuid_string': uuid_string, 'error': str(e),
+      error_class = error.__class__.__name__
+      if( error_class is not None ):
+        return {'log_uuid_string': uuid_string, 'error': str(error),
+                'error_class': error_class, 'core_return': None, 'new_credential': None}
+      return {'log_uuid_string': uuid_string, 'error': str(error),
               'core_return': None, 'new_credential': None}
     if( valid == '' ):
       return True
     return False
 
+  def GetVersion(self):
+    return __version__
+
   def Serve(self, server_name=None, port=None):
     """Main server function
 
@@ -350,6 +410,11 @@ class Server(object):
       server_name: name of server you wish to create
       port: listening port number of server
     """
+    if( not os.path.exists ):
+      try:
+        open(self.log_file, 'w').close()
+      except:
+        raise ServerError('Could not write to logfile "%s"' % self.log_file)
     if( not port ):
       port = self.port
     if( not server_name ):
@@ -364,6 +429,7 @@ class Server(object):
     self.server.register_function(self.CoreRun)
     self.server.register_function(self.GetCredentials)
     self.server.register_function(self.IsAuthenticated)
+    self.server.register_function(self.GetVersion)
     try:
       while 1:
         self.server.handle_request()
