@@ -36,7 +36,7 @@ Most of code is from an example at: http://code.activestate.com/recipes/496786/
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.15'
+__version__ = '0.16'
 
 
 import BaseHTTPServer
@@ -80,7 +80,7 @@ class SecureXMLRpcRequestHandler(SimpleXMLRPCServer.SimpleXMLRPCRequestHandler):
       data = self.rfile.read(int(self.headers["content-length"]))
       response = self.server._marshaled_dispatch(
           data, getattr(self, '_dispatch', None))
-    except:
+    except Exception:
       self.send_response(500)
       self.end_headers()
     else:
